"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavSource = void 0;
const xstream_1 = require("xstream");
const react_native_navigation_1 = require("react-native-navigation");
class NavSource {
    constructor() {
        this._topBar = xstream_1.default.create();
        this._back = xstream_1.default.create();
        this._didAppear = xstream_1.default.create();
        this._didDisappear = xstream_1.default.create();
        this._globalDidAppear = xstream_1.default.create({
            start(listener) {
                this.handle = react_native_navigation_1.Navigation.events().registerComponentDidAppearListener((event) => {
                    listener.next(event);
                });
            },
            stop() {
                var _a, _b;
                (_b = (_a = this.handle) === null || _a === void 0 ? void 0 : _a.remove) === null || _b === void 0 ? void 0 : _b.call(_a);
            },
        });
        this._globalDidDisappear = xstream_1.default.create({
            start(listener) {
                this.handle = react_native_navigation_1.Navigation.events().registerComponentDidDisappearListener((event) => {
                    listener.next(event);
                });
            },
            stop() {
                var _a, _b;
                (_b = (_a = this.handle) === null || _a === void 0 ? void 0 : _a.remove) === null || _b === void 0 ? void 0 : _b.call(_a);
            },
        });
    }
    topBarButtonPress(buttonId) {
        if (buttonId)
            return this._topBar.filter((id) => id === buttonId);
        else
            return this._topBar;
    }
    backPress() {
        return this._back;
    }
    didAppear() {
        return this._didAppear;
    }
    didDisappear() {
        return this._didDisappear;
    }
    globalDidAppear(componentName) {
        if (componentName) {
            return this._globalDidAppear.filter((ev) => ev.componentName === componentName);
        }
        else {
            return this._globalDidAppear;
        }
    }
    globalDidDisappear(componentName) {
        if (componentName) {
            return this._globalDidDisappear.filter((ev) => ev.componentName === componentName);
        }
        else {
            return this._globalDidDisappear;
        }
    }
}
exports.NavSource = NavSource;
//# sourceMappingURL=NavSource.js.map