"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xstream_1 = require("xstream");
const react_1 = require("react");
const react_native_1 = require("react-native");
const react_native_navigation_1 = require("react-native-navigation");
const react_2 = require("@cycle/react");
const NavSource_1 = require("./NavSource");
const clarify = require("clarify-error");
function neverComplete(stream) {
    return xstream_1.default.merge(stream, xstream_1.default.never());
}
function makeComponent(main, // (so: So & MoreSources) => Si & MoreSinks,
engine, screenId) {
    return () => {
        class NavComponent extends react_1.Component {
            constructor(props) {
                super(props);
                this.state = { source: null, sink: null };
                this.backHandler = this.onBackPressed.bind(this);
                this.latestOpts = {};
                this.navEventsSub = react_native_navigation_1.Navigation.events().bindComponent(this);
                this.popped = false;
            }
            componentDidMount() {
                var _a;
                const thisId = this.props.componentId;
                const source = new react_2.ReactSource();
                source._props$._n(this.props);
                const navSource = (this.navSource = new NavSource_1.NavSource());
                const sources = Object.assign(Object.assign({}, engine.sources), { screen: source, navigation: navSource, props: xstream_1.default.of(this.props).compose(neverComplete).remember() });
                const sinks = main(sources);
                this.disposeRun = engine.run(sinks);
                const sink = (_a = sinks.screen) !== null && _a !== void 0 ? _a : this.state.sink;
                if (sinks.navigation) {
                    this.commandSub = sinks.navigation.subscribe({
                        next: (cmd) => {
                            var _a;
                            try {
                                if (cmd.type === 'setRoot') {
                                    react_native_navigation_1.Navigation.setRoot(cmd.layout);
                                    return;
                                }
                                const id = (_a = cmd.id) !== null && _a !== void 0 ? _a : thisId;
                                switch (cmd.type) {
                                    case 'push':
                                        react_native_navigation_1.Navigation.push(id, cmd.layout);
                                        break;
                                    case 'pop':
                                        // Quick hack to fix a backpressure problem.
                                        //
                                        // The sinks.navigation may emit duplicate pops (because the
                                        // user double taps) while the Navigation.pop is still in
                                        // progress (it's async, returns a Promise). Ideally we
                                        // would have a pull system that waits for these commands to
                                        // finish before handling the next command.
                                        if (!this.popped) {
                                            this.popped = true;
                                            react_native_navigation_1.Navigation.pop(id, cmd.options);
                                        }
                                        break;
                                    case 'popTo':
                                        react_native_navigation_1.Navigation.popTo(id);
                                        break;
                                    case 'popToRoot':
                                        react_native_navigation_1.Navigation.popToRoot(id);
                                        break;
                                    case 'showModal':
                                        react_native_navigation_1.Navigation.showModal(cmd.layout);
                                        break;
                                    case 'dismissModal':
                                        react_native_navigation_1.Navigation.dismissModal(id);
                                        break;
                                    case 'dismissAllModals':
                                        react_native_navigation_1.Navigation.dismissAllModals();
                                        break;
                                    case 'setStackRoot':
                                        react_native_navigation_1.Navigation.setStackRoot(id, cmd.layout);
                                        break;
                                    case 'showOverlay':
                                        react_native_navigation_1.Navigation.showOverlay(cmd.layout);
                                        break;
                                    case 'dismissOverlay':
                                        react_native_navigation_1.Navigation.dismissOverlay(id);
                                        break;
                                    case 'mergeOptions':
                                        react_native_navigation_1.Navigation.mergeOptions(id, Object.assign(Object.assign({}, this.latestOpts), cmd.opts));
                                        break;
                                }
                            }
                            catch (err) {
                                console.error(clarify(err, `cycle-native-navigation sink command "${cmd.type}" failed`));
                            }
                        },
                    });
                }
                if (sinks.navOptions) {
                    this.navOptionsSub = sinks.navOptions.subscribe({
                        next: (opts) => {
                            try {
                                this.latestOpts = Object.assign(Object.assign({}, this.latestOpts), opts);
                                react_native_navigation_1.Navigation.mergeOptions(thisId, this.latestOpts);
                            }
                            catch (err) {
                                console.error(clarify(err, `cycle-native-navigation sink "navOptions" failed`));
                            }
                        },
                    });
                }
                react_native_1.BackHandler.addEventListener('hardwareBackPress', this.backHandler);
                this.setState({ source, sink });
            }
            componentDidAppear() {
                if (!this.navSource)
                    return;
                this.navSource._didAppear._n(null);
            }
            render() {
                const { source, sink } = this.state;
                if (!source || !sink)
                    return null;
                return react_1.createElement(react_2.ScopeContext.Provider, { value: source._scope }, react_1.createElement(react_2.StreamRenderer, { stream: sink }));
            }
            componentDidUpdate(prevProps) {
                if (!this.state.source)
                    return;
                if (this.props === prevProps)
                    return;
                this.state.source._props$._n(this.props);
            }
            onBackPressed() {
                if (!this.navSource)
                    return false;
                if (this.navSource._back._ils.length === 0)
                    return false;
                this.navSource._back._n(null);
                return true;
            }
            navigationButtonPressed({ buttonId }) {
                if (!this.navSource)
                    return;
                this.navSource._topBar._n(buttonId);
            }
            componentDidDisappear() {
                if (!this.navSource)
                    return;
                this.navSource._didDisappear._n(null);
            }
            componentWillUnmount() {
                if (this.disposeRun)
                    this.disposeRun();
                if (this.commandSub)
                    this.commandSub.unsubscribe();
                if (this.navOptionsSub)
                    this.navOptionsSub.unsubscribe();
                this.navEventsSub.remove();
                react_native_1.BackHandler.removeEventListener('hardwareBackPress', this.backHandler);
                this.disposeRun = undefined;
                this.latestOpts = undefined;
            }
        }
        return NavComponent;
    };
}
exports.default = makeComponent;
//# sourceMappingURL=makeComponent.js.map